import { AnyObject } from './any-object';
/**
 * Functions to operate on objects
 */
export declare class ObjectOperations {
    /**
     * Returns whether the two given objects are equal
     * @param object1 {object} - First object in the comparison
     * @param object2 {object} - Second object in the comparison
     * @param excludeList {string[]} - List of property names to exclude during the comparison
     */
    AreObjectsEqual(object1: object, object2: object, ...excludeList: string[]): boolean;
    /**
     * Clones the given object
     * @param source {T} - Source object
     * @param excludeList {string[]} - List of property names to exclude from the clone
     */
    CloneObject<T extends AnyObject>(source: T, ...excludeList: string[]): T;
    /**
     * Copies the value of a property from one object to another
     * @param source {object} - Source object
     * @param target {object} - Target object
     * @param propertyName {string} - Property name
     * @param append {boolean} - Indicates whether to add the property to the target object if it doesn't exist
     */
    CopyProperty(source: any, target: any, propertyName: string, append?: boolean): void;
    /**
     * Returns the value of a given property from a given object
     * @param source {object} - Object from which to get the value
     * @param propertyName {string} - Property name from which to get the value
     * @param defaultValue {T} - Default to use if the property doesn't exist or is undefined
     */
    GetProperty<T>(source: any, propertyName: string, defaultValue?: T): T | undefined;
    /**
     * Indicates whether the given value matches the given pattern.
     * @param pattern {string} - Pattern
     * @param value {string} - Value
     */
    IsMatch(pattern: string, value: string): boolean;
    /**
     * Determines whether or not the given value is null or empty
     * @param value {object | string | undefined | null} Value to check if null or empty
     */
    IsNullOrEmpty(value: object | string | undefined | null): boolean;
    /**
     * Determines whether or not the given value is null or whitespace
     * @param value {string | undefined | null} Value to check if null or whitespace
     */
    IsNullOrWhitespace(value: string | undefined | null): boolean;
    /**
     * Returns whether the value of a given property is the same on two objects
     * @param item1 {object} - First object in the comparison
     * @param item2 {object} - Second object in the comparison
     * @param propertyName {string} - Property name
     */
    IsPropertyEqual(item1: any, item2: any, propertyName: string): boolean;
    /**
     * Returns a new object that is the combination of the supplied source objects.
     * @param object1 {object} - First object.
     * @param object2 {object} - Second object.
     */
    MergeObjects<T extends AnyObject>(object1: T, object2: T): T;
    /**
     * Sets the value for the given property on the target based upon the given sources.
     * @param target {object} - Target object
     * @param source1 {object} - Source object 1.
     * @param source2 {object} - Source object 2.
     * @param propertyName {string} - Property name.
     */
    MergeObjectProperty<T extends AnyObject>(target: T, source1: T, source2: T, propertyName: string): void;
    /**
     * Returns a new object based on the given source omitting undefined values
     * @param source {T} - Source object
     */
    RemoveUndefinedElements<T extends AnyObject>(source: T): T;
    /**
     * Sets the value of a given property on the given object
     * @param target {object} - Object to target
     * @param propertyName {string} - Property name to target
     * @param value {any} - Value to set
     * @param append {boolean} - Indicates whether to add the property to the target object if it doesn't exist
     */
    SetProperty(target: any, propertyName?: string, value?: any, append?: boolean): void;
    /**
     * Converts to string
     * @param value {any} Value to convert to string
     */
    ToString(value: any): string;
}
